@@ -17,10 +17,11 @@ import functools
 import os
 import subprocess
 import tempfile
+import unittest
 
 from oslo_config import cfg
+from oslo_utils import strutils
 import six
-import unittest2
 import yaml
 
 import cinderlib
@@ -40,7 +41,8 @@ def set_backend(func, new_name, backend_name):
 def test_all_backends(cls):
     """Decorator to run tests in a class for all available backends."""
     config = BaseFunctTestCase.ensure_config_loaded()
-    for fname, func in cls.__dict__.items():
+    # Prevent dictionary changed size during iteration on Python 3
+    for fname, func in dict(vars(cls)).items():
         if fname.startswith('test_'):
             for backend in config['backends']:
                 bname = backend['volume_backend_name']
@@ -50,12 +52,19 @@ def test_all_backends(cls):
     return cls
 
 
-class BaseFunctTestCase(unittest2.TestCase):
+def get_bool_env(param_string, default=False):
+    param = os.environ.get(param_string, default)
+    return strutils.bool_from_string(param, strict=True)
+
+
+class BaseFunctTestCase(unittest.TestCase):
     FNULL = open(os.devnull, 'w')
     CONFIG_FILE = os.environ.get('CL_FTEST_CFG', '/etc/cinder/cinder.conf')
     PRECISION = os.environ.get('CL_FTEST_PRECISION', 0)
-    LOGGING_ENABLED = os.environ.get('CL_FTEST_LOGGING', False)
+    LOGGING_ENABLED = get_bool_env('CL_FTEST_LOGGING', False)
     ROOT_HELPER = os.environ.get('CL_FTEST_ROOT_HELPER', 'sudo')
+    MEMORY_PERSISTENCE = get_bool_env('CL_FTEST_MEMORY_PERSISTENCE', True)
+    DEFAULT_POOL = os.environ.get('CL_FTEST_POOL_NAME', None)
     tests_config = None
 
     @classmethod
@@ -66,9 +75,16 @@ class BaseFunctTestCase(unittest2.TestCase):
                 cls.tests_config = cinder_to_yaml.convert(cls.CONFIG_FILE)
             else:
                 with open(cls.CONFIG_FILE, 'r') as f:
-                    cls.tests_config = yaml.load(f)
+                    cls.tests_config = yaml.safe_load(f)
             cls.tests_config.setdefault('logs', cls.LOGGING_ENABLED)
             cls.tests_config.setdefault('size_precision', cls.PRECISION)
+
+            backend = cls.tests_config['backends'][0]
+            if backend['volume_driver'].endswith('.RBDDriver'):
+                print('Cinderlib tests use config: %s' % cls.tests_config)
+                ceph_conf = open(backend['rbd_ceph_conf'], 'r').read()
+                print('Contents of ceph.conf are: %s' % ceph_conf)
+
         return cls.tests_config
 
     @staticmethod
@@ -90,6 +106,15 @@ class BaseFunctTestCase(unittest2.TestCase):
                         disable_logs=not config['logs'],
                         persistence_config={'storage': 'memory_db'})
 
+        if cls.MEMORY_PERSISTENCE:
+            # Now replace it with the memory plugin for the tests to ensure the
+            # Cinder driver is compatible with the persistence plugin
+            # mechanism, as the DB plugin could hide issues.
+            cinderlib.Backend.global_initialization = False
+            cinderlib.setup(root_helper=cls.ROOT_HELPER,
+                            disable_logs=not config['logs'],
+                            persistence_config={'storage': 'memory'})
+
         # Initialize backends
         cls.backends = [cinderlib.Backend(**cfg) for cfg in
                         config['backends']]
@@ -125,9 +150,8 @@ class BaseFunctTestCase(unittest2.TestCase):
                     try:
                         vol.detach()
                     except Exception as exc:
-                        errors.append('Error detaching %s for volume %s %s: '
-                                      '%s' % (vol.local_attach.path, vol.id,
-                                              exc))
+                        errors.append('Error detaching %s for volume %s: %s' %
+                                      (vol.local_attach.path, vol.id, exc))
 
                 # Disconnect any existing connections
                 for conn in vol.connections:
@@ -157,6 +181,7 @@ class BaseFunctTestCase(unittest2.TestCase):
 
         vol_size = kwargs.setdefault('size', 1)
         name = kwargs.setdefault('name', backend.id)
+        kwargs.setdefault('pool_name', self.DEFAULT_POOL)
 
         vol = backend.create_volume(**kwargs)
 
